import 'dotenv/config';
import { cac } from 'cac';
import consola from 'consola';
import { APIv1, APIv2, APIVariable, APILicense, fetchAPI, fetchVariable, parsev1, parsev2, generateAxis, parseVariable, parseLicenses } from 'google-font-metadata';
import colors from 'picocolors';
import { intro, group, text, multiselect, select, cancel, confirm, outro } from '@clack/prompts';
import fs from 'fs-extra';
import stringify from 'json-stringify-pretty-compact';
import * as path from 'pathe';
import path__default from 'pathe';
import { generateFontFace } from '@fontsource-utils/generate';
import PQueue from 'p-queue';
import flatten from 'flat';
import got from 'got';

var version$1 = "0.1.2";

const makeFontDownloadPath = (fontDir, fontId, subset, weight, style, extension) => `${fontDir}/files/${fontId}-${subset}-${weight}-${style}.${extension}`;
const makeVariableFontDownloadPath = (fontDir, fontId, subset, axes, style) => `${fontDir}/files/${fontId}-${subset}-${axes.toLowerCase()}-${style}.woff2`;
const makeFontFilePath = (fontId, subset, weight, style, extension) => `./files/${fontId}-${subset}-${weight}-${style}.${extension}`;
const makeVariableFontFilePath = (fontId, subset, axes, style) => `./files/${fontId}-${subset}-${axes}-${style}.woff2`;
const findClosest = (arr, num) => {
  const indexArr = arr.map((weight) => Math.abs(Number(weight) - num));
  const min = Math.min(...indexArr);
  const closest = arr[indexArr.indexOf(min)];
  return closest;
};
const licenseMap = {
  "apache license, version 2.0": "Apache-2.0",
  "sil open font license, 1.1": "OFL-1.1",
  "ubuntu font license, 1.0": "UFL-1.0",
  "mit license": "MIT"
};
const licenseShort = (license) => licenseMap[license.toLowerCase()];

const changelog = `# Changelog
See the [Fontsource CHANGELOG.md](https://github.com/fontsource/fontsource/blob/main/CHANGELOG.md).`;

var version = "4.5.0";

const template = ({ id, family, license, type }, isVariable, oldVersion) => ({
  name: isVariable ? `@fontsource-variable/${id}` : `@fontsource/${id}`,
  version: oldVersion ?? version,
  description: `Self-host the ${family} font in a neatly bundled NPM package.`,
  main: "index.css",
  publishConfig: { access: "public" },
  keywords: [
    "fontsource",
    "font",
    "font family",
    "google fonts",
    id,
    family,
    "css",
    "sass",
    "front-end",
    "web",
    "typeface",
    "variable"
  ],
  author: type === "other" ? license.attribution : "Google Inc.",
  license: license.type,
  homepage: `https://fontsource.org/fonts/${id}`,
  repository: {
    type: "git",
    url: "https://github.com/fontsource/font-files.git",
    directory: `fonts/${isVariable ? "variable" : type}/${id}`
  }
});
const packageJson = async (metadata, opts) => {
  let oldVersion;
  try {
    await fs.access(path.join(opts.dir, "package.json"));
    oldVersion = JSON.parse(
      await fs.readFile(path.join(opts.dir, "package.json"), "utf8")
    ).version;
  } catch {
  }
  const file = template(metadata, opts.isVariable, oldVersion);
  await fs.writeFile(path.join(opts.dir, "package.json"), stringify(file));
};

const variableText = () => `## Variable Fonts

This particular typeface supports [variable fonts](https://developer.mozilla.org/en-US/docs/Web/CSS/CSS_Fonts/Variable_Fonts_Guide).
Variable documentation can be found [here](https://fontsource.org/docs/variable-fonts).

`;
const readme = ({
  id,
  family,
  version,
  weights,
  styles,
  subsets,
  variable,
  license
}, isVariable) => `# Fontsource ${family}

[![npm (scoped)](https://img.shields.io/npm/v/@fontsource/${id}?color=brightgreen)](https://www.npmjs.com/package/@fontsource/${id}) [![Generic badge](https://img.shields.io/badge/fontsource-passing-brightgreen)](https://github.com/fontsource/fontsource) [![Monthly downloads](https://badgen.net/npm/dm/@fontsource/${id})](https://github.com/fontsource/fontsource) [![Total downloads](https://badgen.net/npm/dt/@fontsource/${id})](https://github.com/fontsource/fontsource) [![GitHub stars](https://img.shields.io/github/stars/fontsource/fontsource.svg?style=social&label=Star)](https://github.com/fontsource/fontsource/stargazers)

The CSS and web font files to easily self-host the \u201C${family}\u201D font. Please visit the main [Fontsource website](https://fontsource.org/fonts/${id}) to view more details on this package.

## Quick Installation

Fontsource has a variety of methods to import CSS, such as using a bundler like Webpack. Alternatively, it supports SASS. Full documentation can be found [here](https://fontsource.org/docs/introduction).

\`\`\`javascript
npm install ${isVariable ? `@fontsource-variable/${id}` : `@fontsource/${id}`}
\`\`\`

Within your app entry file or site component, import it in.

\`\`\`javascript
${isVariable ? `import "@fontsource-variable/${id}"; // Defaults to wght axis
import "@fontsource-variable/${id}/wght.css"; // Specify axis
import "@fontsource-variable/${id}/wght-italic.css"; // Specify axis and style` : `import "@fontsource/${id}"; // Defaults to weight 400
import "@fontsource/${id}/400.css"; // Specify weight
import "@fontsource/${id}/400-italic.css"; // Specify weight and style`}

\`\`\`

Supported variables:
- Weights: \`[${weights}]\`
- Styles: \`[${styles}]\`
- Subsets: \`[${subsets}]\`${isVariable ? `
- Axes: \`[${Object.keys(variable).filter((axis) => axis !== "ital")}]\`` : ""}

Finally, you can reference the font name in a CSS stylesheet, CSS Module, or CSS-in-JS.

\`\`\`css
body {
  font-family: "${family};
}
\`\`\`

${variable ? variableText() : ""}## Licensing
It is important to always read the license for every font that you use.
Most of the fonts in the collection use the SIL Open Font License, v1.1. Some fonts use the Apache 2 license. The Ubuntu fonts use the Ubuntu Font License v1.0.

${license.attribution}
[${license.type}](${license.url})

## Other Notes
Font version (provided by source): \`${version}\`.
Feel free to star and contribute new ideas to this repository that aim to improve the performance of font loading, as well as expanding the existing library we already have. Any suggestions or ideas can be voiced via an [issue](https://github.com/fontsource/fontsource/issues).`;

const packagerCustom = async (metadata) => {
  const { id, family, subsets, weights, styles } = metadata;
  const dir = `./${id}`;
  const indexWeight = findClosest(weights, 400);
  for (const subset of subsets) {
    const cssSubset = [];
    const cssSubsetItalic = [];
    for (const weight of weights) {
      for (const style of styles) {
        const fontObj = {
          family,
          style,
          display: "swap",
          weight,
          src: [
            {
              url: makeFontFilePath(id, subset, weight, style, "woff2"),
              format: "woff2"
            },
            {
              url: makeFontFilePath(id, subset, weight, style, "woff"),
              format: "woff"
            }
          ],
          comment: `${id}-${subset}-${weight}-${style} CUSTOM`,
          displayVar: true
        };
        const css = generateFontFace(fontObj);
        if (style === "normal") {
          await fs.writeFile(path__default.join(dir, `${subset}-${weight}.css`), css);
          cssSubset.push(css);
        } else {
          await fs.writeFile(path__default.join(dir, `${subset}-${weight}-${style}.css`), css);
          cssSubsetItalic.push(css);
        }
      }
      if (weight === indexWeight) {
        await fs.writeFile(path__default.join(dir, "index.css"), cssSubset.join("\n\n"));
      }
      await fs.writeFile(path__default.join(dir, `${subset}.css`), cssSubset.join("\n\n"));
      if (cssSubsetItalic.length > 0) {
        await fs.writeFile(path__default.join(dir, `${subset}-italic.css`), cssSubsetItalic.join("\n\n"));
      }
    }
  }
};

const buildCustom = async (metadata) => {
  const dir = `./${metadata.id}`;
  fs.ensureDirSync(dir);
  fs.ensureDirSync(path__default.join(dir, "/files"));
  await packagerCustom(metadata);
  await fs.writeFile(path__default.join(dir, "README.md"), readme(metadata));
  await fs.writeFile(path__default.join(dir, "metadata.json"), stringify(metadata));
  await fs.writeFile(path__default.join(dir, "unicode.json"), stringify({}));
  await fs.writeFile(path__default.join(dir, "CHANGELOG.md"), changelog);
  await packageJson(metadata, dir);
};

const create = async () => {
  intro(colors.cyan(colors.bold("fontsource")));
  const cfg = await group(
    {
      name: () => text({
        message: colors.bold("What is the name of the font?"),
        placeholder: "Noto Sans JP",
        validate(value) {
          if (!value)
            return "Please enter a name";
        }
      }),
      subsets: () => text({
        message: colors.bold("What are the subsets of the font? (separate by commas)"),
        placeholder: "latin, latin-ext, japanese",
        validate(value) {
          if (!value)
            return "Please enter at least one subset";
        }
      }),
      weights: () => text({
        message: colors.bold("What are the weights of the font? (separate by commas)"),
        placeholder: "100, 200, 300, 400, 500, 600, 700, 800, 900",
        validate(value) {
          if (!value)
            return "Please enter at least one weight";
          const weights = value.split(",").map(Number);
          if (weights.some((weight) => Number.isNaN(weight)))
            return "Please enter only numbers";
        }
      }),
      styles: () => multiselect({
        message: colors.bold("What are the styles of the font?"),
        options: [
          { value: "normal", label: "Normal" },
          { value: "italic", label: "Italic" }
        ],
        required: true
      }),
      version: () => text({
        message: colors.bold("What is the version of the font?"),
        placeholder: "v1.0",
        validate(value) {
          if (!value)
            return "Please enter a version";
        }
      }),
      category: () => select({
        message: colors.bold("What is the category of the font?"),
        options: [
          { value: "sans-serif", label: "Sans Serif" },
          { value: "serif", label: "Serif" },
          { value: "display", label: "Display" },
          { value: "monospace", label: "Monospace" },
          { value: "handwriting", label: "Handwriting" },
          { value: "icons", label: "Icons" },
          { value: "other", label: "Other" }
        ]
      }),
      license: () => select({
        message: colors.bold("What is the license of the font?"),
        options: [
          { value: "sil open font license, 1.1", label: "SIL Open Font License (OFL-1.1)" },
          { value: "apache license, version 2.0", label: "Apache License 2.0 (Apache-2.0)" },
          { value: "ubuntu font license, 1.0", label: "Ubuntu Font License (UFL-1.0)" },
          { value: "mit", label: "MIT License" },
          { value: "other", label: "Other License", hint: "Please make an issue verifying if this is usable! We are likely to reject PRs that do not match the above licenses." }
        ]
      }),
      licenseUrl: () => text({
        message: colors.bold("What is the URL of the license?"),
        validate(value) {
          if (!value)
            return "Please enter a URL";
        }
      }),
      licenseAttribution: () => text({
        message: colors.bold("What is the author name or attribution of the font?"),
        validate(value) {
          if (!value)
            return "Please enter a name or attribution";
        }
      }),
      sourceUrl: () => text({
        message: colors.bold("What is the URL of the source files? GitHub repositories are preferred"),
        validate(value) {
          if (!value)
            return "Please enter a URL";
        }
      })
    },
    {
      onCancel: () => {
        cancel("Package creation cancelled");
        process.exit(0);
      }
    }
  );
  const metadata = {
    id: cfg.name.toLowerCase().replace(/ /g, "-"),
    family: cfg.name,
    subsets: cfg.subsets.split(",").map((subset) => subset.trim()),
    weights: cfg.weights.split(",").map((weight) => Number.parseInt(weight.trim(), 10)),
    styles: cfg.styles,
    defSubset: cfg.subsets.split(",")[0].trim(),
    variable: false,
    lastModified: (/* @__PURE__ */ new Date()).toISOString(),
    version: cfg.version,
    category: cfg.category,
    license: {
      type: licenseShort(cfg.license) ?? cfg.license,
      url: cfg.licenseUrl,
      attribution: cfg.licenseAttribution
    },
    source: cfg.sourceUrl,
    type: "other"
  };
  const confirmCreate = await confirm({
    message: colors.bold(`Create package in current directory? ./${metadata.id}`)
  });
  if (!confirmCreate) {
    cancel("Package creation cancelled.");
    process.exit(0);
  }
  try {
    await buildCustom(metadata);
  } catch (error) {
    cancel("Package creation failed.");
    consola.error(error);
    return;
  }
  outro(colors.green(`You're all set!

	Now copy all the font files into the files directory and run "fontsource create-verify" to verify your package.

	Please ensure the file names match the format "${metadata.id}-subset-weight-style.extension"
	Example: "${metadata.id}-latin-400-normal.woff2" or "${metadata.id}-latin-ext-700-italic.woff"

	Please also copy the appropriate LICENSE file from your license URL to the root directory of the package.`));
};

const verifyFilenames = async (metadata, dir) => {
  const filenames = await fs.readdir(path__default.join(dir, "files"));
  if (filenames.length === 0) {
    throw new Error("No files found in the files directory.");
  }
  const expectedFilenames = [];
  for (const subset of metadata.subsets) {
    for (const weight of metadata.weights) {
      for (const style of metadata.styles) {
        expectedFilenames.push(`${metadata.id}-${subset}-${weight}-${style}.woff2`, `${metadata.id}-${subset}-${weight}-${style}.woff`);
      }
    }
  }
  const missingFilenames = expectedFilenames.filter((filename) => !filenames.includes(filename));
  if (missingFilenames.length > 0) {
    throw new Error(`The following files are missing: ${missingFilenames.join(", ")}

	Please ensure the file names match the format "${metadata.id}-subset-weight-style.extension"
	Example: "${metadata.id}-latin-400-normal.woff2" or "${metadata.id}-latin-ext-700-italic.woff"`);
  }
  const nonMatchingFilenames = filenames.filter((filename) => !expectedFilenames.includes(filename));
  if (nonMatchingFilenames.length > 0) {
    throw new Error(`The following files are not expected: ${nonMatchingFilenames.join(", ")}

	Please ensure the file names match the format "${metadata.id}-subset-weight-style.extension"
	Example: "${metadata.id}-latin-400-normal.woff2" or "${metadata.id}-latin-ext-700-italic.woff"`);
  }
};
const verify = async () => {
  intro(colors.cyan(colors.bold("fontsource")));
  const cfg = await group({
    id: () => text({
      message: colors.bold("What is the ID of the font?"),
      placeholder: "noto-sans-jp",
      validate(value) {
        if (!value)
          return "Please enter an ID.";
      }
    })
  });
  if (!await fs.pathExists(`./${cfg.id}/files`)) {
    cancel("The directory does not exist.");
    return;
  }
  if (!await fs.pathExists(`./${cfg.id}/metadata.json`)) {
    cancel("The metadata.json file does not exist.");
    return;
  }
  if (!await fs.pathExists(`./${cfg.id}/LICENSE`)) {
    cancel("The LICENSE file does not exist.");
    return;
  }
  try {
    const dir = `./${cfg.id}`;
    const metadata = await fs.readJson(path__default.join(dir, "metadata.json"));
    await verifyFilenames(metadata, dir);
  } catch (error) {
    cancel(error.message);
    return;
  }
  outro(colors.green("All checks passed! Feel free to send a PR over to the main repo adding the package to the appropriate fonts directory."));
};

const gotDownload = async (url, dest) => {
  const response = await got(url).buffer();
  await fs.writeFile(dest, response);
};
const pairGenerator = (variants) => {
  const flattenedPairs = Object.entries(flatten(variants));
  const splitPairs = flattenedPairs.map((pair) => [
    pair[0].split("."),
    pair[1]
  ]);
  const ABSOLUTE_URL_REGEX = /^https?:\/\//;
  const urlPairs = splitPairs.filter(
    (pair) => ABSOLUTE_URL_REGEX.test(pair[1].toString())
  );
  const cleanedPairs = urlPairs.filter((pair) => {
    const extension = pair[0][4];
    if (extension === "truetype" || extension === "opentype") {
      return false;
    }
    return true;
  });
  return cleanedPairs;
};
const generateLinks = (id, opts) => {
  const fontV1 = APIv1[id];
  const fontV2 = APIv2[id];
  let downloadURLPairsV1 = pairGenerator(fontV1.variants);
  const downloadURLPairsV2 = pairGenerator(fontV2.variants);
  let hasUnicodeSubsets = false;
  const re = /\[.*?]/g;
  for (const pair of downloadURLPairsV2) {
    if (re.test(pair[0][2])) {
      hasUnicodeSubsets = true;
    }
  }
  if (!hasUnicodeSubsets) {
    downloadURLPairsV1 = downloadURLPairsV1.filter(
      (pair) => pair[0][4] !== "woff2"
    );
  }
  const linksV1 = downloadURLPairsV1.map((pair) => {
    const types = pair[0];
    const dest = makeFontDownloadPath(
      opts.dir,
      id,
      types[2],
      Number(types[0]),
      types[1],
      types[4]
    );
    const url = pair[1];
    return {
      url,
      dest
    };
  });
  const linksV2 = downloadURLPairsV2.map((pair) => {
    const types = pair[0];
    const dest = makeFontDownloadPath(
      opts.dir,
      id,
      types[2].replace("[", "").replace("]", ""),
      Number(types[0]),
      types[1],
      types[4]
    );
    const url = pair[1];
    return { url, dest };
  });
  const links = [...linksV1, ...linksV2];
  return links;
};
const variableLinks = (id, opts) => {
  const fontVariable = APIVariable[id];
  const downloadURLPairsVariable = pairGenerator(fontVariable.variants);
  const links = downloadURLPairsVariable.map((pair) => {
    const types = pair[0];
    const dest = makeVariableFontDownloadPath(
      opts.dir,
      id,
      types[2].replace("[", "").replace("]", ""),
      types[0],
      types[1]
    );
    const url = pair[1];
    return {
      url,
      dest
    };
  });
  return links;
};
const queue$1 = new PQueue({ concurrency: 60 });
const download = async (id, opts) => {
  await fs.ensureDir(path.join(opts.dir, "files"));
  const links = opts.isVariable ? variableLinks(id, opts) : generateLinks(id, opts);
  for (const link of links) {
    queue$1.add(() => gotDownload(link.url, link.dest));
  }
  await queue$1.onIdle();
};

const BASE_URL = "https://cdn.jsdelivr.net/gh/google/fonts@main/";
const apache = (id) => `apache/${id}/LICENSE.txt`;
const ofl = (id) => `ofl/${id}/OFL.txt`;
const ufl = (id) => `ufl/${id}/UFL.txt`;
const generateLicense = async (id, licenseType, opts) => {
  const licensePath = path.join(opts.dir, "LICENSE");
  const googleId = id.replace(/-/g, "");
  let licenseUrl;
  switch (licenseType.toLowerCase()) {
    case "apache license, version 2.0": {
      licenseUrl = `${BASE_URL}${apache(googleId)}`;
      break;
    }
    case "sil open font license, 1.1": {
      licenseUrl = `${BASE_URL}${ofl(googleId)}`;
      break;
    }
    case "ubuntu font license, 1.0": {
      licenseUrl = `${BASE_URL}${ufl(googleId)}`;
      break;
    }
    default: {
      throw new Error(`Unknown license type: ${licenseType}`);
    }
  }
  try {
    const { body } = await got.get(licenseUrl);
    await fs.writeFile(licensePath, body);
  } catch (error) {
    throw new Error(`Unable to download license for ${id}
${error}`);
  }
};

const packagerV1 = async (id, opts) => {
  const { family, styles, weights, subsets, variants } = APIv1[id];
  for (const subset of subsets) {
    const cssSubset = [];
    const cssSubsetItalic = [];
    for (const weight of weights) {
      for (const style of styles) {
        if (style in variants[weight]) {
          const fontObj = {
            family,
            style,
            display: "swap",
            weight,
            src: [
              {
                url: makeFontFilePath(id, subset, weight, style, "woff2"),
                format: "woff2"
              },
              {
                url: makeFontFilePath(id, subset, weight, style, "woff"),
                format: "woff"
              }
            ],
            comment: `${id}-${subset}-${weight}-${style}`,
            displayVar: true
          };
          const css = generateFontFace(fontObj);
          if (style === "normal") {
            const cssPath = path.join(opts.dir, `${subset}-${weight}.css`);
            await fs.writeFile(cssPath, css);
            cssSubset.push(css);
          } else {
            const cssStylePath = path.join(
              opts.dir,
              `${subset}-${weight}-${style}.css`
            );
            await fs.writeFile(cssStylePath, css);
            cssSubsetItalic.push(css);
          }
        }
      }
    }
    const cssSubsetPath = path.join(opts.dir, `${subset}.css`);
    await fs.writeFile(cssSubsetPath, cssSubset.join("\n\n"));
    if (cssSubsetItalic.length > 0) {
      const cssSubsetItalicPath = path.join(opts.dir, `${subset}-italic.css`);
      await fs.writeFile(cssSubsetItalicPath, cssSubsetItalic.join("\n\n"));
    }
  }
};

const packagerV2 = async (id, opts) => {
  const { family, styles, weights, variants, unicodeRange } = APIv2[id];
  const indexWeight = findClosest(weights, 400);
  const unicodeKeys = Object.keys(unicodeRange);
  for (const weight of weights) {
    for (const style of styles) {
      const cssStyle = [];
      for (const subset of unicodeKeys) {
        if (style in variants[weight]) {
          const fontObj = {
            family,
            style,
            display: "swap",
            weight,
            unicodeRange: unicodeRange[subset],
            src: [
              {
                url: makeFontFilePath(
                  id,
                  subset.replace("[", "").replace("]", ""),
                  // Remove brackets from unicode subset
                  weight,
                  style,
                  "woff2"
                ),
                format: "woff2"
              },
              {
                url: makeFontFilePath(id, "all", weight, style, "woff"),
                format: "woff"
              }
            ],
            comment: `${id}-${subset}-${weight}-${style}`,
            displayVar: true
          };
          const css = generateFontFace(fontObj);
          cssStyle.push(css);
        }
      }
      if (style in variants[weight]) {
        if (style === "normal") {
          const cssPath = path.join(opts.dir, `${weight}.css`);
          await fs.writeFile(cssPath, cssStyle.join("\n\n"));
          if (weight === indexWeight) {
            await fs.writeFile(
              path.join(opts.dir, "index.css"),
              cssStyle.join("\n\n")
            );
          }
        } else {
          const cssStylePath = path.join(opts.dir, `${weight}-${style}.css`);
          await fs.writeFile(cssStylePath, cssStyle.join("\n\n"));
        }
      }
    }
  }
};

const packagerVariable = async (id, opts) => {
  const font = APIv2[id];
  const fontVariable = APIVariable[id];
  let indexCSS = "";
  for (const axes of Object.keys(fontVariable.variants)) {
    const variant = fontVariable.variants[axes];
    const styles = Object.keys(variant);
    const axesLower = axes.toLowerCase();
    const variableOpts = {
      wght: fontVariable.axes.wght
    };
    if (axes === "standard" || axes === "full" || axes === "wdth")
      variableOpts.stretch = fontVariable.axes.wdth;
    if (axes === "standard" || axes === "full" || axes === "slnt")
      variableOpts.slnt = fontVariable.axes.slnt;
    for (const style of styles) {
      const cssStyle = [];
      for (const subset of Object.keys(variant[style])) {
        const fontObj = {
          family: font.family,
          style,
          display: "swap",
          weight: findClosest(font.weights, 400),
          unicodeRange: font.unicodeRange[subset],
          variable: variableOpts,
          src: [
            {
              url: makeVariableFontFilePath(id, subset, axesLower, style),
              format: "woff2"
            }
          ],
          comment: `${id}-${subset}-${axesLower}-${style}`,
          displayVar: true
        };
        const css2 = generateFontFace(fontObj);
        cssStyle.push(css2);
      }
      const filename = style === "normal" ? `${axesLower}.css` : `${axesLower}-${style}.css`;
      const cssPath = path.join(opts.dir, filename);
      const css = cssStyle.join("\n\n");
      await fs.writeFile(cssPath, css);
      if (axes === "wght")
        indexCSS = css;
      if (!indexCSS && axes === "opsz")
        indexCSS = css;
    }
  }
  await fs.writeFile(path.join(opts.dir, "index.css"), indexCSS);
};

const build = async (id, opts) => {
  const font = APIv2[id];
  const fontVariable = APIVariable[id];
  const fontLicense = APILicense[id];
  await fs.mkdir(opts.dir, { recursive: true });
  let changed = false;
  try {
    await fs.access(path.join(opts.dir, "metadata.json"));
    const metadata = JSON.parse(
      await fs.readFile(path.join(opts.dir, "metadata.json"), "utf8")
    );
    changed = metadata.lastModified !== font.lastModified;
  } catch {
    changed = true;
  }
  if (changed || opts.force) {
    try {
      const pkgJsonPath = path.join(opts.dir, "package.json");
      const tempPath = path.join(
        opts.tmpDir,
        opts.isVariable ? `${id}-variable.json` : `${id}.json`
      );
      await fs.access(pkgJsonPath);
      await fs.copy(pkgJsonPath, tempPath);
      await fs.emptyDir(opts.dir);
      await fs.copy(tempPath, pkgJsonPath);
      await fs.remove(tempPath);
    } catch {
    }
    await download(id, opts);
    if (opts.isVariable) {
      await packagerVariable(id, opts);
    } else {
      await packagerV1(id, opts);
      await packagerV2(id, opts);
    }
    const metadata = {
      id,
      family: font.family,
      subsets: font.subsets,
      weights: font.weights,
      styles: font.styles,
      defSubset: font.defSubset,
      variable: fontVariable ? fontVariable.axes : false,
      lastModified: font.lastModified,
      version: font.version,
      category: font.category,
      license: {
        type: licenseShort(fontLicense.license.type) ?? fontLicense.license.type,
        url: fontLicense.license.url,
        attribution: fontLicense.original
      },
      source: "https://github.com/google/fonts",
      type: "google"
    };
    await fs.writeFile(path.join(opts.dir, "README.md"), readme(metadata, opts.isVariable));
    await fs.writeFile(
      path.join(opts.dir, "unicode.json"),
      stringify(font.unicodeRange)
    );
    await fs.writeFile(path.join(opts.dir, "CHANGELOG.md"), changelog);
    await generateLicense(id, fontLicense.license.type, opts);
    await fs.writeFile(
      path.join(opts.dir, "metadata.json"),
      stringify(metadata)
    );
    await packageJson(metadata, opts);
  }
};

const queue = new PQueue({ concurrency: 3 });
queue.on("error", (error) => {
  throw new Error(error);
});
queue.on("idle", async () => {
  consola.success(
    `All ${Object.keys(APIv2).length} Google Fonts have been processed.`
  );
  consola.success(
    `${Object.keys(APIVariable).length} variable fonts have been processed.`
  );
});
const buildPackage = async (id, opts) => {
  consola.info(`Downloading ${id}`);
  const optsNew = {
    ...opts,
    dir: path.join(opts.dir, "google", id)
  };
  await build(id, optsNew);
  consola.success(`Finished processing ${id}`);
};
const buildVariablePackage = async (id, opts) => {
  consola.info(`Downloading ${id} ${colors.bold(colors.yellow("[VARIABLE]"))}`);
  const optsNew = {
    ...opts,
    isVariable: true,
    dir: path.join(opts.dir, "variable", id)
  };
  await build(id, optsNew);
  consola.success(`Finished processing ${id} ${colors.bold(colors.yellow("[VARIABLE]"))}`);
};
const testIds = [
  "akshar",
  "alegreya",
  "archivo",
  "ballet",
  "fraunces",
  "noto-sans-jp",
  "recursive",
  "roboto-flex"
];
const processGoogle = async (opts, fonts) => {
  const outDir = path.resolve(process.cwd(), "fonts");
  await fs.ensureDir(outDir);
  const tmpDir = path.join(outDir, "fontsource_temp_packages");
  await fs.ensureDir(tmpDir);
  let fontIds = fonts;
  if (opts.test) {
    fontIds = testIds;
  } else if (!fonts || fonts.length === 0) {
    fontIds = Object.keys(APIv2);
  }
  for (const id of fontIds) {
    const buildOpts = {
      dir: outDir,
      tmpDir,
      isVariable: false,
      force: opts.force ?? false
    };
    try {
      if (id in APIv2) {
        queue.add(() => buildPackage(id, buildOpts));
        if (APIVariable[id]) {
          queue.add(() => buildVariablePackage(id, buildOpts));
        }
      } else {
        consola.warn(
          `Skipping ${id} as it is not a Google Font.`
        );
      }
    } catch (error) {
      throw new Error(`${id} experienced an error. ${error}`);
    }
  }
  await queue.onIdle();
  await fs.rm(tmpDir, { recursive: true });
};

const cli = cac("fontsource");
cli.command("fetch [key]", "Fetch parsing metadata for all fonts").option("-f, --force", "Force parse all metadata").action(async (key, options) => {
  try {
    const finalKey = key ?? process.env.GOOGLE_FONT_API_KEY;
    if (!finalKey) {
      throw new Error("No API key provided.");
    }
    if (options.force) {
      consola.info(
        `Parsing all metadata... ${colors.bold(colors.red("[FORCE]"))}`
      );
    } else {
      consola.info("Parsing all metadata...");
    }
    await Promise.all([fetchAPI(finalKey), fetchVariable()]);
    await parsev1(options.force, false);
    await parsev2(options.force, false);
    await generateAxis();
    await parseVariable(false);
    await parseLicenses();
  } catch (error) {
    consola.error(error);
  }
});
cli.command("build [...fonts]", "Build font packages").option("-v, --variable", "Only build variable fonts").option("-f, --force", "Force rebuild all packages").option("-t, --test", "Build test fonts only").action(async (fonts, options) => {
  try {
    consola.info(`Building packages... ${options.force ? colors.bold(colors.red("[FORCE]")) : ""}`);
    await processGoogle(options, fonts);
  } catch (error) {
    consola.error(error);
  }
});
cli.command("create").action(async () => {
  try {
    await create();
  } catch (error) {
    consola.error(error);
  }
});
cli.command("create-verify").action(async () => {
  try {
    await verify();
  } catch (error) {
    consola.error(error);
  }
});
cli.help();
cli.version(version$1);
cli.parse();
